local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local GetMapNameByID = _G.GetMapNameByID

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

NS.extensions = {
	[1] = "Classic",
	[2] = "The Burning Crusade",
	[3] = "Wrath of the Lich King",
	[4] = "Cataclysm",
	[5] = "Mists of Pandaria",
}

NS.extensionTranslations = {
	[1] = _G.EXPANSION_NAME0,
	[2] = _G.EXPANSION_NAME1,
	[3] = _G.EXPANSION_NAME2,
	[4] = _G.EXPANSION_NAME3,
	[5] = _G.EXPANSION_NAME4,
}

NS.instances = {
	-- Classic
	[1] = {
		"Blackwing Lair",
		"Blackrock Spire",
		"Molten Core",
		"Ruins of Ahn'Qiraj",
		"Temple of Ahn'Qiraj",
	},
	-- The Burning Crusade
	[2] = {
		"Black Temple",
		"Gruul's Lair",
		"Karazhan",
		"Magtheridon's Lair",
		"Mount Hyjal",
		"Serpentshrine Cavern",
		"Sunwell Plateau",
		"Tempest Keep",
		"Zul'Aman",
	},
	-- Wrath of the Lich King
	[3] = {
		"The Eye of Eternity",
		"Icecrown Citadel",
		"Naxxramas",
		"The Nexus",
		"The Obsidian Sanctum",
		"Onyxia's Lair",
		"The Ruby Sanctum",
		"Ulduar",
		"Vault of Archavon",
	},
	-- Cataclysm
	[4] = {
		"Baradin Hold",
		"The Bastion of Twilight",
		"Blackwing Descent",
		"Dragon Soul",
		"Firelands",
		"Throne of the Four Winds",
	},
	-- Mists of Pandaria
	[5] = {
		"Heart of Fear",
		"Mogu'shan Vaults",
		"Siege of Orgrimmar",
		"Terrace of the Endless Spring",
		"Throne of Thunder",
	},
}

NS.instanceTranslations = {
	-- Classic
	["Blackwing Lair"]                = GetMapNameByID(755) or "Blackwing Lair",
	["Blackrock Spire"]               = GetMapNameByID(696) or "Blackrock Spire",
	["Molten Core"]                   = GetMapNameByID(721) or "Molten Core",
	["Ruins of Ahn'Qiraj"]            = GetMapNameByID(717) or "Ruins of Ahn'Qiraj",
	["Temple of Ahn'Qiraj"]           = GetMapNameByID(766) or "Temple of Ahn'Qiraj",
	-- The Burning Crusade
	["Black Temple"]                  = GetMapNameByID(796) or "Black Temple",
	["Gruul's Lair"]                  = GetMapNameByID(776) or "Gruul's Lair",
	["Karazhan"]                      = GetMapNameByID(799) or "Karazhan",
	["Magtheridon's Lair"]            = GetMapNameByID(779) or "Magtheridon's Lair",
	["Mount Hyjal"]                   = GetMapNameByID(775) or "Mount Hyjal",
	["Serpentshrine Cavern"]          = GetMapNameByID(780) or "Serpentshrine Cavern",
	["Sunwell Plateau"]               = GetMapNameByID(789) or "Sunwell Plateau",
	["Tempest Keep"]                  = GetMapNameByID(782) or "Tempest Keep",
	["Zul'Aman"]                      = GetMapNameByID(781) or "Zul'Aman",
	-- Wrath of the Lich King
	["The Eye of Eternity"]           = GetMapNameByID(527) or "The Eye of Eternity",
	["Icecrown Citadel"]              = GetMapNameByID(604) or "Icecrown Citadel",
	["Naxxramas"]                     = GetMapNameByID(535) or "Naxxramas",
	["The Obsidian Sanctum"]          = GetMapNameByID(531) or "The Obsidian Sanctum",
	["Onyxia's Lair"]                 = GetMapNameByID(718) or "Onyxia's Lair",
	["The Ruby Sanctum"]              = GetMapNameByID(609) or "The Ruby Sanctum",
	["Ulduar"]                        = GetMapNameByID(529) or "Ulduar",
	["Trial of the Crusader"]         = GetMapNameByID(543) or "Trial of the Crusader",
	["Vault of Archavon"]             = GetMapNameByID(532) or "Vault of Archavon",
	-- Cataclysm
	["Baradin Hold"]                  = GetMapNameByID(752) or "Baradin Hold",
	["The Bastion of Twilight"]       = GetMapNameByID(758) or "The Bastion of Twilight",
	["Blackwing Descent"]             = GetMapNameByID(754) or "Blackwing Descent",
	["Dragon Soul"]                   = GetMapNameByID(824) or "Dragon Soul",
	["Firelands"]                     = GetMapNameByID(800) or "Firelands",
	["Throne of the Four Winds"]      = GetMapNameByID(773) or "Throne of the Four Winds",
	-- Mists of Pandaria
	["Heart of Fear"]                 = GetMapNameByID(897) or "Heart of Fear",
	["Mogu'shan Vaults"]              = GetMapNameByID(896) or "Mogu'shan Vaults",
	["Siege of Orgrimmar"]            = GetMapNameByID(953) or "Siege of Orgrimmar",
	["Terrace of the Endless Spring"] = GetMapNameByID(886) or "Terrace of the Endless Spring",
	["Throne of Thunder"]             = GetMapNameByID(930) or "Throne of Thunder",
}

NS.instanceList = {}

do
	for extension, instances in ipairs(NS.instances) do
		for _, instance in ipairs(instances) do
			NS.instanceList[instance] = extension
		end
	end
end

function NS:TranslateExpansion(id)
	return NS.extensionTranslations[id] or ""
end

function NS:TranslateInstance(instance)
	if type(instance) ~= "string" then
		return ""
	end

	return NS.instanceTranslations[instance] or instance
end

function NS:IsValidInstance(instance)
	return NS.instanceTranslations[instance] and true or false
end

function NS:IterateInstances()
	return pairs(NS.instanceList)
end
