local _G = _G

local pairs = pairs

-- addon name and namespace
local ADDON, NS = ...

-- queue functions
local function PushLeft(self, entry)
	self._first = self._first - 1		
	self._queue[self._first] = entry
end

local function PushRight(self, entry)
	self._last = self._last + 1		
	self._queue[self._last] = entry
end

local function PopLeft(self, entry)
	if self._first > self._last then
		return
	end
	
	local value = self._queue[self._first]
	self._queue[self._first] = nil
	self._first = self._first + 1
	
	return value
end

local function PopRight(self, entry)
	if self._first > self._last then
		return
	end
	
	local value = self._queue[self._last]
	self._queue[self._last] = nil
	self._last = self._last - 1
	
	return value
end

local function PeekLeft(self)
	return self._queue[self._first]
end

local function PeekRight(self)
	return self._queue[self._last]
end

local function Clear(self)
	while self._last >= self._first do
		self._queue[self._last] = nil
		self._last = self._last - 1
	end
end

local function Count(self) 
	return self._last - self._first + 1
end

local function IsEmpty(self) 
	return self._first > self._last
end

local function Contains(self, value) 
	for index = self._first, self._last do
		if self._queue[index] == value then
			return true
		end
	end

	return false
end

-- factory method for queue creation
-- NOTE: it is possible to push nil into the queue
--       in that case make sure to check IsEmpty before popping values 
--       else you cannot distinguish a failed pop of an empty queue from a valid return of nil
function NS:CreateQueue()
	local queue = {
		-- members
		_queue = {},
		_first =  0,
		_last  = -1,
		
		-- functions
		PushLeft    = PushLeft,
		PushRight   = PushRight,
		PopLeft     = PopLeft,
		PopRight    = PopRight,
		PeekLeft    = PeekLeft,
		PeekRight   = PeekRight,
		Clear       = Clear,
		Count       = Count,
		IsEmpty     = IsEmpty,
		Contains    = Contains,
	}
	
	return queue
end

function NS:CreateFIFOQueue()
	local fifo = {
		-- members
		_queue = {},
		_first =  0,
		_last  = -1,
		
		-- functions
		Enqueue     = PushRight,
		Dequeue     = PopLeft,
		Peek        = PeekLeft,
		Clear       = Clear,
		Count       = Count,
		IsEmpty     = IsEmpty,
		Contains    = Contains,
	}
	
	return fifo
end

function NS:CreateStack()
	local stack = {
		-- members
		_queue = {},
		_first =  0,
		_last  = -1,
		
		-- functions
		Push        = PushRight,
		Pop         = PopRight,
		Peek        = PeekRight,
		Clear       = Clear,
		Count       = Count,
		IsEmpty     = IsEmpty,
		Contains    = Contains,
	}
	
	return stack
end
